@@ -33,7 +33,7 @@ fn get_function_signature(function_prototype: &str) -> [u8; 4] {
 /// `#[ewasm_main(auto)]`
 /// Auto unwrap the output of the result object from ewasm_main function.
 /// This is for a scenario that you are using a rust non-rust client,
-/// and you are only care the happy case of excuting the contract.
+/// and you are only care the happy case of executing the contract.
 ///
 /// ```compile_fail
 /// #[ewasm_main]
@@ -60,32 +60,43 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
 
     let output_type = match input.sig.clone().output {
         syn::ReturnType::Type(_, boxed) => match Box::into_inner(boxed) {
-            syn::Type::Path(syn::TypePath { path: p, .. }) => match p.segments.first() {
-                Some(syn::PathSegment {
-                    arguments:
-                        syn::PathArguments::AngleBracketed(syn::AngleBracketedGenericArguments {
-                            args: a,
+            syn::Type::Path(syn::TypePath { path: p, .. }) => {
+                let mut ok_type: Option<String> = None;
+                let mut segments = p.segments.clone();
+                while let Some(pair) = segments.pop() {
+                    ok_type = match pair.into_value() {
+                        syn::PathSegment {
+                            arguments:
+                                syn::PathArguments::AngleBracketed(
+                                    syn::AngleBracketedGenericArguments { args: a, .. },
+                                ),
                             ..
-                        }),
-                    ..
-                }) => match a.first() {
-                    Some(a) => match a {
-                        syn::GenericArgument::Type(syn::Type::Path(syn::TypePath {
-                            path: p,
-                            ..
-                        })) => {
-                            if let Some(syn::PathSegment { ident: i, .. }) = p.segments.first() {
-                                Some(i.to_string())
-                            } else {
-                                None
-                            }
-                        }
+                        } => match a.first() {
+                            Some(a) => match a {
+                                syn::GenericArgument::Type(syn::Type::Path(syn::TypePath {
+                                    path: p,
+                                    ..
+                                })) => {
+                                    if let Some(syn::PathSegment { ident: i, .. }) =
+                                        p.segments.last()
+                                    {
+                                        Some(i.to_string())
+                                    } else {
+                                        None
+                                    }
+                                }
+                                _ => None,
+                            },
+                            _ => None,
+                        },
                         _ => None,
-                    },
-                    _ => None,
-                },
-                _ => None,
-            },
+                    };
+                    if ok_type.is_some() {
+                        break;
+                    }
+                }
+                ok_type
+            }
             _ => None,
         },
         _ => None,
@@ -100,6 +111,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
             #[cfg(all(not(target_arch = "wasm32"), not(test)))]
             pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
             #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
             #[no_mangle]
             pub fn main() {
                 #input
@@ -124,6 +136,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
             #[cfg(all(not(target_arch = "wasm32"), not(test)))]
             pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
             #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
             #[no_mangle]
             pub fn main() {
                 #input
@@ -150,6 +163,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
             #[cfg(all(not(target_arch = "wasm32"), not(test)))]
             pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
             #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
             #[no_mangle]
             pub fn main() {
                 #input
@@ -170,6 +184,7 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
             #[cfg(all(not(target_arch = "wasm32"), not(test)))]
             pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
             #[cfg(target_arch = "wasm32")]
+            #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
             #[no_mangle]
             pub fn main() {
                 #input
@@ -240,11 +255,37 @@ pub fn ewasm_fn(_attr: TokenStream, item: TokenStream) -> TokenStream {
     let result = quote! {
         pub(crate) const #sig_name : [u8; 4] = [#sig_0, #sig_1, #sig_2, #sig_3];
         #[cfg(target_arch = "wasm32")]
+        #[cfg(not(any(feature = "constructor", feature = "constructor-test")))]
         #input
     };
     result.into()
 }
 
+/// helps you to build your constructor for the contract
+#[proc_macro_error]
+#[proc_macro_attribute]
+pub fn ewasm_constructor(_attr: TokenStream, item: TokenStream) -> TokenStream {
+    let input = syn::parse_macro_input!(item as syn::ItemFn);
+    let name = &input.sig.ident;
+    if name.to_string() != "constructor" {
+        abort!(input.sig.ident, "please name the function as `constructor`");
+    }
+    let result = quote! {
+        #[cfg(target_arch = "wasm32")]
+        #[cfg(any(feature = "constructor", feature = "constructor-test"))]
+        #[no_mangle]
+        #input
+
+        #[cfg(target_arch = "wasm32")]
+        #[cfg(feature = "constructor-test")]
+        #[no_mangle]
+        pub fn main() {
+            #name();
+        }
+    };
+    result.into()
+}
+
 /// helps you to build your handler in other module
 ///
 /// This macro will automatically generated as `{FUNCTION_NAME}_SIG`
@@ -602,9 +643,26 @@ pub fn derive_value(item: TokenStream) -> TokenStream {
 /// assert!(default_input != default_person_input)
 /// ```
 #[cfg(feature = "rdb")]
-#[proc_macro_derive(Table)]
+#[proc_macro_derive(Table, attributes(belongs_to, belongs_none_or))]
 pub fn derive_table(item: TokenStream) -> TokenStream {
     let input = syn::parse_macro_input!(item as syn::DeriveInput);
+    let attrs = &input.attrs;
+    let mut belongs_to: Option<String> = None;
+    for a in attrs.iter() {
+        let syn::Attribute { path, tokens, .. } = a;
+        let attr_name = path.segments.first().map(|s| s.ident.to_string());
+        if Some("belongs_to".to_string()) == attr_name {
+            belongs_to = Some(
+                tokens
+                    .to_string()
+                    .strip_prefix('(')
+                    .expect("#[belongs_to(table_name)] is not correct")
+                    .strip_suffix(')')
+                    .expect("#[belongs_to(table_name)] is not correct")
+                    .to_string(),
+            );
+        }
+    }
     let struct_name = &input.ident;
     let fields_with_type = match &input.data {
         syn::Data::Struct(syn::DataStruct {
@@ -649,16 +707,15 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
 
     let protocol_name = Ident::new(&format!("{}Protocol", struct_name), Span::call_site());
     let wrapper_name = Ident::new(&format!("{}Wrapper", struct_name), Span::call_site());
-    let captal_mod_name = Ident::new(
+    let captal_name = Ident::new(
         &format!("{}", struct_name).to_ascii_uppercase(),
         Span::call_site(),
     );
-    let lower_mod_name = Ident::new(
+    let lower_name = Ident::new(
         &format!("{}", struct_name).to_ascii_lowercase(),
         Span::call_site(),
     );
-
-    quote!(
+    let mut output = quote!(
         impl sewup::rdb::traits::Record for #struct_name {}
 
         #[derive(Clone, sewup::Serialize, sewup::Deserialize)]
@@ -727,7 +784,7 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
                 }
             }
         }
-        pub mod #captal_mod_name {
+        pub mod #captal_name {
             use sewup_derive::ewasm_fn_sig;
             pub(crate) const GET_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::get());
             pub(crate) const CREATE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::create());
@@ -755,14 +812,14 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
             }
         }
         #[cfg(target_arch = "wasm32")]
-        pub mod #lower_mod_name {
+        pub mod #lower_name {
             use super::*;
             use sewup::primitives::IntoEwasmAny;
             pub type Protocol = #protocol_name;
             pub type Wrapper = #wrapper_name;
-            pub type _Instance = #struct_name;
+            pub type _InstanceType = #struct_name;
             pub fn get(proc: Protocol) -> sewup::Result<sewup::primitives::EwasmAny> {
-                let table = sewup::rdb::Db::load(None)?.table::<_Instance>()?;
+                let table = sewup::rdb::Db::load(None)?.table::<_InstanceType>()?;
                 if proc.filter {
                     let mut raw_output: Vec<Wrapper> = Vec::new();
                     for r in table.all_records()?.drain(..){
@@ -807,21 +864,21 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
                 }
             }
             pub fn create(proc: Protocol) -> sewup::Result<sewup::primitives::EwasmAny> {
-                let mut table = sewup::rdb::Db::load(None)?.table::<_Instance>()?;
+                let mut table = sewup::rdb::Db::load(None)?.table::<_InstanceType>()?;
                 let mut output_proc = proc.clone();
                 output_proc.records[0].id = Some(table.add_record(proc.records[0].into())?);
                 table.commit()?;
                 Ok(output_proc.into_ewasm_any())
             }
             pub fn update(proc: Protocol) -> sewup::Result<sewup::primitives::EwasmAny> {
-                let mut table = sewup::rdb::Db::load(None)?.table::<_Instance>()?;
+                let mut table = sewup::rdb::Db::load(None)?.table::<_InstanceType>()?;
                 let id = proc.records[0].id.unwrap_or_default();
                 table.update_record(id, Some(proc.records[0].clone().into()))?;
                 table.commit()?;
                 Ok(proc.into_ewasm_any())
             }
             pub fn delete(proc: Protocol) -> sewup::Result<sewup::primitives::EwasmAny> {
-                let mut table = sewup::rdb::Db::load(None)?.table::<_Instance>()?;
+                let mut table = sewup::rdb::Db::load(None)?.table::<_InstanceType>()?;
                 let id = proc.records[0].id.unwrap_or_default();
                 table.update_record(id, None)?;
                 table.commit()?;
@@ -829,15 +886,15 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
             }
         }
         #[cfg(not(target_arch = "wasm32"))]
-        pub mod #lower_mod_name {
+        pub mod #lower_name {
             use super::*;
             pub type Protocol = #protocol_name;
             pub type Wrapper = #wrapper_name;
-            pub type _Instance = #struct_name;
+            pub type _InstanceType = #struct_name;
             pub type Query = Wrapper;
 
             #[inline]
-            pub fn protocol(instance: _Instance) -> Protocol {
+            pub fn protocol(instance: _InstanceType) -> Protocol {
                 instance.into()
             }
             impl Protocol {
@@ -849,7 +906,7 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
                     true
                 }
             }
-            pub fn query(instance: _Instance) -> Wrapper {
+            pub fn query(instance: _InstanceType) -> Wrapper {
                 instance.into()
             }
 
@@ -863,7 +920,27 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
                 }
             }
         }
-    ).into()
+    ).to_string();
+
+    if let Some(parent_table) = belongs_to {
+        let lower_parent_table = &format!("{}", &parent_table).to_ascii_lowercase();
+        let parent_table = Ident::new(&parent_table, Span::call_site());
+        let lower_parent_table_ident = Ident::new(&lower_parent_table, Span::call_site());
+        let field_name = &format!("{}_id", lower_parent_table);
+
+        output += &quote! {
+            impl #struct_name {
+                pub fn #lower_parent_table_ident (&self) -> sewup::Result<#parent_table> {
+                    let id: usize = sewup::utils::get_field_by_name(self, #field_name);
+                    let parent_table = sewup::rdb::Db::load(None)?.table::<#parent_table>()?;
+                    parent_table.get_record(id)
+                }
+            }
+        }
+        .to_string();
+    }
+
+    output.parse().unwrap()
 }
 
 /// helps you setup the test mododule, and test cases in contract.
@@ -881,7 +958,7 @@ pub fn derive_table(item: TokenStream) -> TokenStream {
 /// The test runtime will be create in the module, and all the test case will use the same test
 /// runtime, if you can create more runtimes for testing by setup more test modules.
 /// You can setup a log file when running the test as following, then use `sewup::ewasm_dbg!` to debug the
-/// ewasm contract in the excuting in the runtime.
+/// ewasm contract in the executing in the runtime.
 /// ```compile_fail
 /// #[ewasm_test(log=/path/to/logfile)]
 /// mod tests {
@@ -921,10 +998,11 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
                 use std::process::Command;
                 use std::sync::Arc;
 
-                fn _build_wasm() -> String {
+                fn _build_wasm(opt: Option<String>) -> String {
+                    let cargo_cmd = format!("cargo build --release --target=wasm32-unknown-unknown {}", opt.unwrap_or_default());
                     let output = Command::new("sh")
                         .arg("-c")
-                        .arg("cargo build --release --target=wasm32-unknown-unknown")
+                        .arg(&cargo_cmd)
                         .output()
                         .expect("failed to build wasm binary");
                     if !output.status.success() {
@@ -948,23 +1026,32 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
                     Arc<RefCell<TestRuntime>>,
                     impl Fn(Arc<RefCell<TestRuntime>>, &str, [u8; 4], Option<&[u8]>, Vec<u8>) -> (),
                 ) {
-                    (
-                        Arc::new(RefCell::new(TestRuntime::default()"#
-            .to_string()
-            + &runtime_log_option
-            + r#")),
+                    let rt = Arc::new(RefCell::new(TestRuntime::default()"#.to_string()
+                            + &runtime_log_option
+                            + r#"));
+                    let mut h = ContractHandler {
+                        call_data: None,
+                        rt: Some(rt.clone())
+                    };
+
+                    match h.run_fn(_build_wasm(Some("--features=constructor-test".to_string())), None, 1_000_000_000_000) {
+                        Ok(_) => (),
+                        Err(e) => {
+                            panic!("vm run constructor error: {:?}", e);
+                        }
+                    };
+
+                    (rt,
                         |runtime: Arc<RefCell<TestRuntime>>,
                          fn_name: &str,
                          sig: [u8; 4],
                          input_data: Option<&[u8]>,
                          expect_output: Vec<u8>| {
                             let mut h = ContractHandler {
-                                call_data: Some(_build_wasm()),
-                                ..Default::default()
+                                call_data: Some(_build_wasm(None)),
+                                rt: Some(runtime.clone())
                             };
 
-                            h.rt = Some(runtime.clone());
-
                             match h.execute(sig, input_data, 1_000_000_000_000) {
                                 Ok(r) => assert_eq!(r.output_data, expect_output, "{} output is unexpected", fn_name),
                                 Err(e) => {
@@ -976,8 +1063,13 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
                 }
 
                 #[test]
-                fn _compile_test() {
-                    _build_wasm();
+                fn _compile_runtime_test() {
+                    _build_wasm(None);
+                }
+
+                #[test]
+                fn _compile_constructor_test() {
+                    _build_wasm(Some("--features=constructor-test".to_string()));
                 }"#;
         return mod_re
             .replace(&context, &template)
